from typing import Optional
from typing import TypedDict

from testlodge._types import DateTimeStr
from testlodge._types import Pagination


class SuiteJSON(TypedDict):

    id: int
    name: str
    plan_id: Optional[int]
    project_id: int
    created_at: DateTimeStr
    updated_at: DateTimeStr


class SuiteListJSON(TypedDict):

    pagination: Pagination
    suites: list[SuiteJSON]



from types import SimpleNamespace

import requests
from furl import furl
from furl.furl import furl as Url
from requests import Response
from testlodge._types import Identifier
from testlodge.api.base import BaseAPI as API
from testlodge.api.suite_section import SuiteSectionAPI


class Client:
    """Represents a client accessing the TestLodge API."""

    def __init__(
        self,
        email: str,
        api_key: str,
        account_id: Identifier,
        apis: list[type[API]] = None,
    ):

        if apis is None:
            apis = [SuiteSectionAPI]

        self.email: str = email
        self.api_key: str = api_key
        self.account_id: Identifier = account_id

        self.history: list[Response] = []

        # Initialize the APIs
        self.api = SimpleNamespace()
        for api in apis:
            api_instance = api(client=self)
            setattr(self.api, api.name, api_instance)

    @property
    def base_url(self) -> Url:

        return furl(f'https://api.testlodge.com/v1/account/{self.account_id}')

    def _request(self, method: str, url: str, *args, **kwargs) -> Response:
        """Wrap requests.request to add handlers for response,
        logging, etc."""

        response = requests.request(
            method=method,
            url=url,
            auth=(self.email, self.api_key),
            *args,  # type: ignore
            **kwargs,  # type: ignore
        )

        self.history.append(response)

        return response



from typing import TypedDict

from testlodge._types import DateTimeStr
from testlodge._types import Pagination


class SuiteSectionJSON(TypedDict):

    id: int
    title: str
    suite_id: int
    created_at: DateTimeStr
    updated_at: DateTimeStr


class SuiteSectionListJSON(TypedDict):

    pagination: Pagination
    suite_sections: list[SuiteSectionJSON]



from typing import Optional
from typing import TypedDict

from testlodge._types import DateTimeStr
from testlodge._types import Pagination
from testlodge.models.custom_field import CustomFieldJSON
from testlodge.models.requirements import RequirementDocumentJSON
from testlodge.models.user import UserJSON


class CaseJSON(TypedDict):

    id: int
    project_id: int
    suite_section_id: int
    position: int
    last_saved_by_id: int
    last_saved_by: UserJSON
    created_at: DateTimeStr
    updated_at: DateTimeStr
    custom_fields: list[CustomFieldJSON]
    requirements: list[RequirementDocumentJSON]
    step_number: str
    title: str
    description: Optional[str]
    test_steps: Optional[str]
    expected_result: Optional[str]


class CaseListJSON(TypedDict):

    pagination: Pagination
    steps: list[CaseJSON]



from typing import TypedDict

from testlodge._types import DateTimeStr
from testlodge._types import Pagination


class RequirementDocumentJSON(TypedDict):

    id: int
    title: str
    should_version: bool
    project_id: int
    created_at: DateTimeStr
    updated_at: DateTimeStr


class RequirementDocumentListJSON(TypedDict):

    pagination: Pagination
    requirement_documents: list[RequirementDocumentJSON]



from testlodge.api.suite_section import SuiteSectionAPI
from testlodge.client import Client
from testlodge.models.case import CaseJSON
from testlodge.models.custom_field import CustomFieldJSON
from testlodge.models.requirements import RequirementDocumentJSON
from testlodge.models.suite_section import SuiteSectionJSON
from testlodge.models.user import UserJSON


__all__ = [
    # Client
    'Client',
    # Models - User
    'UserJSON',
    # Models - Requirement Documents
    'RequirementDocumentJSON',
    # Models - Custom Field
    'CustomFieldJSON',
    # Models - Suite
    # 'SuiteJSON',
    # Models - Suite Section
    'SuiteSectionJSON',
    # Models - Case
    'CaseJSON',
    # API - Suite Section
    'SuiteSectionAPI',
]
